import glob
import logging
import os

import gdown


def get_file_from_gdrive(url: str, folder_name: str, cache_dir: str) -> None:
    cached_datasets = list(map(os.path.basename, glob.glob(f"{cache_dir}*")))

    if folder_name not in cached_datasets:  # Check if file is already downloaded in cache
        gdown.download_folder(url, output=cache_dir, quiet=True, use_cookies=False)
    else:
        logging.debug(f"Using cached version of {folder_name}")



from abc import ABC, abstractmethod

# Url of the FOLDER of the dataset in Google Drive. The folder must be publicly available.
GDRIVE_URL = ""


class Dataset(ABC):
    """
    Set of methods that every dataset should implement.

    Apart from these methods the class can
    implement whichever other method is considered needed e.g. different versions of the dataset.
    """
    @abstractmethod
    def __init__(self, cache_dir: str | None = '~/.cache/dare-datasets/') -> None:
        """
        Initializer chooses the cache directory where the dataset will be downloaded or retrieved.
        Note the dataset is not yet downloaded.

        Args:
            cache_dir: Folder where the dataset will be downloaded or retrieved from.
        """
        pass

    @abstractmethod
    def _init_data(self):
        """
        Loads the dataset from disk. If the dataset is not available on disk, it is downloaded and saved in cache.
        """
        pass

    @abstractmethod
    def get_info(self) -> dict[str, str]:
        """
        Returns a dictionary with information about the dataset.
        Necessary:
            - **name**: name of dataset
            - **description**: description of dataset
            - **url**: url of gdrive file
            - **original_url**: original url of dataset (can be the paper url)
            - **formats**: list of formats the dataset is available eg. ["csv", "json", "xml"]

        """
        pass

    @abstractmethod
    def get_raw(self):
        """
        Returns the raw data of the dataset on whichever format we consider default.
        Structure (if not applicable it is not enforced):
        ```python
            {
                 "train":[],
                 "dev":[],
                 "test":[]
            }
        ```
        """
        pass

    @abstractmethod
    def get_processed(self):
        """
        Returns the processed data of the dataset on whichever format we consider default.
        If many processed versions are available, the one we consider default should be the one returned.
        Structure (if not applicable it is not enforced):
        ```python
            {
                 "train":[],
                 "dev":[],
                 "test":[]
            }
        ```
        """
        pass



import glob
import json
import os
import typing
from pathlib import Path

from dare_datasets.dataset_abc import Dataset
from dare_datasets.utils.downloader import get_file_from_gdrive

GDRIVE_URL = "https://drive.google.com/drive/folders/1LCD4gObeX-PBxkNYNuZxSmrTBvzEK095?usp=sharing"
DATASET_NAME = "QR2T_Benchmark"


class QR2TBenchmark(Dataset):
    def __init__(self, cache_dir: typing.Optional[str] = f'{Path.home()}/.cache/dare-datasets/') -> None:
        self.data = None
        self.cache_dir = cache_dir
        self.dataset_folder = cache_dir + DATASET_NAME + "/"

    def get_info(self) -> dict[str, str]:
        return {
            "name": "QR2T Benchmark",
            "description": "QR2T Benchmark is an extension of the Spider dataset for the QR2T task.",
            "url": GDRIVE_URL,
            "original_url": "-",
            "formats": ["json"],
            "dataset_folder": self.dataset_folder if self.data is not None else "NOT_YET_DOWNLOADED"
        }

    def _init_data(self):
        get_file_from_gdrive(url=GDRIVE_URL, cache_dir=self.cache_dir, folder_name=DATASET_NAME)

        self.data = {}
        for file in glob.glob(self.dataset_folder + "*"):
            self.data[os.path.basename(file).split('.')[0]] = json.load(open(file))

    def get_raw(self):
        if self.data is None:
            self._init_data()

        return self.data

    def get_processed(self):
        pass
